#ifndef __CCHROMOSOME_H__
#define __CCHROMOSOME_H__

#include "Geom.h"

class CGene {
public:
	int			m_PolyIdx;
	int			m_Seq;
	float		m_XPos;

	CGene()
	{
		m_PolyIdx = 0;
		m_Seq  = 0;
		m_XPos = 0;
	}
};

class CChromosome {
public:
	CGene		*m_pGenes;
	int			m_nGenes;
	float		m_FitnessValue;

	CChromosome( )
	{
		m_pGenes = NULL;
		m_nGenes = 0;
		m_FitnessValue = 0;
	}

	~CChromosome()
	{
		if(m_pGenes) delete [] m_pGenes;
		m_nGenes = 0;
	}

	CChromosome( const CChromosome &Other)
	{
		m_pGenes = new CGene[ Other.m_nGenes ];
		memcpy(m_pGenes,Other.m_pGenes, sizeof(CGene)* Other.m_nGenes);
		m_nGenes = Other.m_nGenes;
		m_FitnessValue = Other.m_FitnessValue;
	}

	CChromosome& operator = (const CChromosome &Other )
	{
		if( m_pGenes ) delete [] m_pGenes;
		m_pGenes = new CGene[ Other.m_nGenes ];
		memcpy(m_pGenes,Other.m_pGenes, sizeof(CGene)* Other.m_nGenes);
		m_nGenes = Other.m_nGenes;
		m_FitnessValue = Other.m_FitnessValue;
		return (*this);
	}

	void SetSize( int Size )
	{
		if( m_pGenes ) delete [] m_pGenes;

		m_pGenes = new CGene[ Size ];
		m_nGenes = Size;
		for(int i=0;i<m_nGenes;++i) m_pGenes[i].m_PolyIdx = i;
	}

	void CrossOver( CChromosome &Other, CChromosome *pOutput, int Children );
	bool Mutate( int SeqMin, int SeqMax, float XPosMin, float XPosMax );	

	void WriteInfo()
	{
		printf("Chromosome Info ---\n");
		printf("nGenes: %d\n",m_nGenes);
		printf("Fitness: %f\n",m_FitnessValue);
	}
};

extern float g_MutationProbability;

#endif