#include "CContainer.h"
#include "CStat.h"

CContainer::~CContainer()
{
}

bool CContainer::AddPolygon( const char *FileName )
{
	CPolygon p;
	
	if( !p.ReadPolygon( FileName ) ){
		printf("Couldn't Load Polygon %s\n",FileName );
		return false;
	}

	m_PolyList.push_back( p );

	return true;
}

void CContainer::ValidateChromosome( CChromosome &c)
{
	for(int i=0;i<c.m_nGenes;++i){
		
		CPolygon &Poly = m_PolyList[ c.m_pGenes[i].m_PolyIdx ];

		if( (Poly.m_BoundingBox.m_MinPoint.m_v[X] + c.m_pGenes[i].m_XPos) < 0 ){
			c.m_pGenes[i].m_XPos = -Poly.m_BoundingBox.m_MinPoint.m_v[X];
		}

		if( (Poly.m_BoundingBox.m_MaxPoint.m_v[X] + c.m_pGenes[i].m_XPos) > m_Width ){
			c.m_pGenes[i].m_XPos = m_Width - Poly.m_BoundingBox.m_MaxPoint.m_v[X];
		}
	}
}


void CContainer::RandomlyGenerateFirstGeneration()
{
	std::vector < int > SeqList;
	

	for(size_t i=0;i<m_ChromosomePool.size();++i){

		CChromosome &c = m_ChromosomePool[i];
		int k;

		SeqList.clear();
		for(size_t i=0;i<m_PolyList.size();++i){
			SeqList.push_back( (int)i );
		};

		for(int n=0;n<c.m_nGenes;++n){
			CGene &g = c.m_pGenes[n];
			g.m_PolyIdx = n;

			k = (int)(rand()% SeqList.size());
			g.m_Seq = SeqList[k];
			
			SeqList.erase( (SeqList.begin() + k) );

			g.m_XPos = ((float)rand() / (float)RAND_MAX) * m_Width;
		}
	}
}

void CContainer::PrepareEverything( unsigned int nFirstPopulationSize )
{
	//Calculate the worst height;	
	m_WorstHeight = 0;
	for(size_t i=0;i<m_PolyList.size();++i){
		m_WorstHeight += m_PolyList[i].m_BoundingBox.GetHeight();
	}

	//Prepare Chromosome Pool
	for(unsigned int i=0;i<nFirstPopulationSize;++i){
		CChromosome c;
		c.SetSize( (int)m_PolyList.size() );
		m_ChromosomePool.push_back(c);
	}
}

int GeneSortCallBack( const void *p0, const void *p1 )
{
	CGene *pGene0 = (CGene*) p0;
	CGene *pGene1 = (CGene*) p1;

	return pGene0->m_Seq - pGene1->m_Seq; 
}

void CContainer::GetCollidablePolygonList( const CPolygon &Poly, const std::vector< CPolygon > &PolyPool, std::vector< CPolygon > &Output )
{
	CBoundingBox PolyBoundingBox;
	PolyBoundingBox = Poly.m_BoundingBox;
	
	PolyBoundingBox.m_MinPoint = PolyBoundingBox.m_MinPoint + Poly.m_Pos;
	PolyBoundingBox.m_MaxPoint = PolyBoundingBox.m_MaxPoint + Poly.m_Pos;
	PolyBoundingBox.m_MinPoint.m_v[Y] = 0;
	
	for(size_t i=0;i<PolyPool.size();++i){

		CBoundingBox Box = PolyPool[i].m_BoundingBox;
		
		Box.m_MinPoint = Box.m_MinPoint + PolyPool[i].m_Pos;
		Box.m_MaxPoint = Box.m_MaxPoint + PolyPool[i].m_Pos;

		if( PolyBoundingBox.IsCollide( Box ) ){
			Output.push_back( PolyPool[i] );
		}
	}
	
}

/*
	The chromosome must be validated before calling this function
*/
float CContainer::EvaluateFitnessValue( const CChromosome &c,std::vector<CPolygon> &DumpedPolyList, bool bDrawDebugPoints )
{
	float MaxHeight = 0;
	float yPoint = 0;
	float xPoint = 0;

	bool  bCollided = false;
	bool  bHasCollision = false;
	float yResult = 0;		
	float yMinResult = 0;
	float yMaxPoint = 0;
	float xMaxPoint = 0;
	bool  bFirstTime = true;
	std::vector< CPolygon > CollidablePolygonList;

	CGene *pGene = new CGene[ c.m_nGenes ];
	memcpy( pGene, c.m_pGenes, c.m_nGenes * sizeof(CGene));

	qsort(pGene,c.m_nGenes,sizeof(CGene),GeneSortCallBack);

	DumpedPolyList.clear();

	for(int i=0;i<c.m_nGenes;++i){

		CPolygon Poly = m_PolyList[ pGene[i].m_PolyIdx ];
		Poly.m_Pos.m_v[Y] = m_WorstHeight * 2;
		Poly.m_Pos.m_v[X] = pGene[i].m_XPos;
		
		bCollided = false;
		yResult = 0;
		bHasCollision = false;		
		bFirstTime = true;

		yMinResult = (Poly.m_BoundingBox.m_MinPoint + Poly.m_Pos ).m_v[Y];
		yMaxPoint  = 0;
		xMaxPoint  = (Poly.m_BoundingBox.m_MinPoint + Poly.m_Pos ).m_v[X];;

		CollidablePolygonList.clear();
		
		GetCollidablePolygonList( Poly, DumpedPolyList,CollidablePolygonList);
		
		//printf("PRUNED %d many polygons\n",DumpedPolyList.size() - CollidablePolygonList.size());

		for(int p=(int)CollidablePolygonList.size()-1;p>=0;--p){
				
			if( (Poly.m_Pos.m_v[Y] - yMinResult + Poly.m_BoundingBox.m_MinPoint[Y]) 
				> 
				((CollidablePolygonList[p].m_BoundingBox.m_MaxPoint + CollidablePolygonList[p].m_Pos).m_v[Y]) ){ 

				//printf("POLYGON PRUNED\n");
				continue;
			}

			bCollided = Poly.GetApproachDistanceFromTop( CollidablePolygonList[p], &yResult, &xPoint, &yPoint );
			
			if( bCollided && !bHasCollision ) bHasCollision = true;

			if(bCollided ){				

				if( yMinResult > yResult ){
					yMinResult = yResult;
					yMaxPoint = yPoint;
					xMaxPoint = xPoint;
				}
			}

		}

		/*
		for(int p=(int)DumpedPolyList.size()-1;p>=0;--p){
				
			if( (Poly.m_Pos.m_v[Y] - yMinResult + Poly.m_BoundingBox.m_MinPoint[Y]) 
				> 
				((DumpedPolyList[p].m_BoundingBox.m_MaxPoint + DumpedPolyList[p].m_Pos).m_v[Y]) ){ 

				printf("POLYGON PRUNED\n");
				continue;
			}

			bCollided = Poly.GetApproachDistanceFromTop( DumpedPolyList[p], &yResult, &xPoint, &yPoint );
			
			if( bCollided && !bHasCollision ) bHasCollision = true;

			if(bCollided ){				

				if( yMinResult > yResult ){
					yMinResult = yResult;
					yMaxPoint = yPoint;
					xMaxPoint = xPoint;
				}
			}

		}
		
		*/

		Poly.m_Pos.m_v[Y] -= yMinResult;

		float HeightReached = (Poly.m_BoundingBox.m_MaxPoint + Poly.m_Pos).m_v[Y];
		
		MaxHeight = HeightReached > MaxHeight ? HeightReached : MaxHeight;

		DumpedPolyList.push_back( Poly );
		
		if( bDrawDebugPoints && m_pDisplay ){
			m_pDisplay->AddDebugPoint(xMaxPoint,yMaxPoint,0); 
		}
	}

	delete [] pGene;

	

	return m_WorstHeight - MaxHeight;
}

int CContainer::RandomEventCore(float p[],int n)
{
	float r = (float)((double)rand() / (double)RAND_MAX) * 100.0f;
	int i;

	float upper_boundary = 0;
	float sum = 0;	

	for(i=0;i<n;++i){
		upper_boundary += p[i] * 100.0f;
		if( r < upper_boundary ) return i;
		else if( abs( upper_boundary - r ) < 0.05 ) return i;
		
	}	

	/* We Should not ideally reach here */
	for(i=0;i<n;++i){
		sum += p[i];
	}	
	printf("P dint exists r=%f,sum=%f upper_boundary=%f\n",r,sum,upper_boundary);

	return -1;
}

int CContainer::RandomEvent(float p[],int n,int ex)
{
	int i,k,ret;
	int c;

	if( ex >= 0 ){
		float *new_p = (float*)malloc( sizeof(float) * (n - 1) );
		float sum = 0;
		
		for(i=0,c=0;i<n;++i){
			
			if( i != ex ){
				new_p[c++] = p[i];
				sum += p[i];
			}
		}

		for(k=0;k<c;++k){
			new_p[k] /= sum;
		}
	
		ret = RandomEventCore( new_p, c );

		free( new_p );

		return ret;
	}
	
	return RandomEventCore( p, n );
}

void CContainer::RunGA( int nFirstPopulationSize, int nGenerations )
{
	std::vector< CPolygon > DumpedPolyList;
	std::vector< float >    ProbabilityDist;
	float					FitnessSum;
	bool					bFirstTime = true;

	m_bStop = false;	

	srand(time(NULL));

#ifdef STAT
	CStatistics::Get()->Reset();
#endif

	PrepareEverything( nFirstPopulationSize );	
	RandomlyGenerateFirstGeneration();

	FitnessSum = 0;
	for(size_t i=0;i<m_ChromosomePool.size();++i){
	
		
		ValidateChromosome( m_ChromosomePool[i] );
		
		FitnessSum += m_ChromosomePool[i].m_FitnessValue = EvaluateFitnessValue( m_ChromosomePool[i], DumpedPolyList );
		
		ProbabilityDist.push_back( m_ChromosomePool[i].m_FitnessValue );
		
		if(bFirstTime){
			bFirstTime = false;
			m_BestFitness = m_WorstFitness = m_ChromosomePool[i].m_FitnessValue;
			m_BestChromosome = m_ChromosomePool[i];
		}
		else{
			if( m_ChromosomePool[i].m_FitnessValue < m_WorstFitness )
				m_WorstFitness = m_ChromosomePool[i].m_FitnessValue;

			if( m_ChromosomePool[i].m_FitnessValue > m_BestFitness ){
				m_BestFitness = m_ChromosomePool[i].m_FitnessValue;
				m_BestChromosome = m_ChromosomePool[i];
			}
		}
	}

	for(size_t i=0;i<ProbabilityDist.size();++i){
		ProbabilityDist[i] = ProbabilityDist[i] / FitnessSum;
	};

	m_AvgFitness = (float)FitnessSum / (float)m_ChromosomePool.size();

	
	OnGenerationDone( 0 );	

	if(m_bStop) return;

	int nMatings;

	for(int g=0;g<nGenerations;++g){

		//nMatings = (int)(m_ChromosomePool.size()/4);
		nMatings = (int)(m_ChromosomePool.size());

		std::vector< CChromosome > NewlyBornList;

		for(int k=0;k<nMatings;++k){
		
			if( m_bStop ) return ;

			int p0 = RandomEvent( &ProbabilityDist[0], (int)ProbabilityDist.size(),-1);
			int p1 = RandomEvent( &ProbabilityDist[0], (int)ProbabilityDist.size(),p0);

			CChromosome NewBorn[2];
			
			/*
			printf("Picked Chromosome 0\n");
			m_ChromosomePool[p0].WriteInfo();
			printf("Picked Chromosome 1\n");
			m_ChromosomePool[p1].WriteInfo();
			*/

			//printf("Picked %d and %d\n",p0,p1);

			if( (p0 < 0) || (p1 < 0) ){
				printf("WRONG (LESSER) idx\n %d , %d\n",p0,p1);
			}

			if( (p0 >= (int)m_ChromosomePool.size()) || (p1>=(int)m_ChromosomePool.size()) ){
				printf("WRONG (GREATER) idx %d , %d\n",p0,p1);
			}

			m_ChromosomePool[p0].CrossOver( m_ChromosomePool[p1], &NewBorn[0], 1 );

#ifdef STAT
	CStatistics::Get()->AddCrossOver( 1 );
#endif
			
			if( NewBorn[0].Mutate(0,(int)m_PolyList.size()-1,0,m_Width) )
			{
#ifdef STAT
				CStatistics::Get()->AddMutation( 1 );
#endif
			}

			//NewBorn[1].Mutate(0,(int)m_PolyList.size()-1,0,m_Width);

			ValidateChromosome( NewBorn[0] );
			//ValidateChromosome( NewBorn[1] );

			DumpedPolyList.clear();
			
			NewBorn[0].m_FitnessValue = EvaluateFitnessValue( NewBorn[0],DumpedPolyList);
			//NewBorn[1].m_FitnessValue = EvaluateFitnessValue( NewBorn[1],DumpedPolyList);

			if(m_bStop) return;

			if( NewBorn[0].m_FitnessValue < m_WorstFitness )
				m_WorstFitness = NewBorn[0].m_FitnessValue;

			
			//if( NewBorn[1].m_FitnessValue < m_WorstFitness )
			//	m_WorstFitness = NewBorn[1].m_FitnessValue;

			

			if( NewBorn[0].m_FitnessValue > m_BestFitness ){
				m_BestFitness = NewBorn[0].m_FitnessValue;
				m_BestChromosome = NewBorn[0];
			}

			
			//if( NewBorn[1].m_FitnessValue > m_BestFitness ){
			//	m_BestFitness = NewBorn[1].m_FitnessValue;
			//	m_BestChromosome = NewBorn[1];
			//}
			

			NewlyBornList.push_back( NewBorn[0] );
			//NewlyBornList.push_back( NewBorn[1] );
		}

		/**
		* For now i will just replace the old generation 
		* with the new one
		*/
		/*
		for(size_t j=0;j<NewlyBornList.size();++j){
			m_ChromosomePool.push_back( NewlyBornList[j]);		
		}
		*/

		if(m_bStop) return;

		m_ChromosomePool = NewlyBornList;

		printf("Newly Created %d\n", NewlyBornList.size());

		NewlyBornList.clear();
		
		if(m_bStop) return;

		/*
		int CullCount = 0;
		std::vector< CChromosome > TmpChromosomePool;

		for(size_t c=0;c<m_ChromosomePool.size();++c){
			CChromosome &Chromosome = m_ChromosomePool[c];
			
			float CullParameter = (Chromosome.m_FitnessValue - m_WorstFitness) / (m_BestFitness - m_WorstFitness);
			
			if( CullParameter >= m_CullThreshold ){
				//m_ChromosomePool.erase( m_ChromosomePool.begin() + c );
				TmpChromosomePool.push_back( Chromosome );
				//CullCount++;
			}
		}

		printf("Culled away %d\n",m_ChromosomePool.size() - TmpChromosomePool.size());

		m_ChromosomePool = TmpChromosomePool;
		*/

		ProbabilityDist.clear();
		FitnessSum = 0;

		for(size_t i=0;i<m_ChromosomePool.size();++i){
			FitnessSum += m_ChromosomePool[i].m_FitnessValue;
			ProbabilityDist.push_back( m_ChromosomePool[i].m_FitnessValue );
		}

		if(m_bStop) return;

		for(size_t i=0;i<m_ChromosomePool.size();++i){
			ProbabilityDist[i] = ProbabilityDist[i] / FitnessSum;
		}

		if(m_bStop) return;

		m_AvgFitness = (float)FitnessSum / (float) m_ChromosomePool.size();

		OnGenerationDone( g+1 );		

		printf("Generation %d has been created with %d many individuals\n", g+1, m_ChromosomePool.size());
		printf("Average fitness= %f\n",m_AvgFitness);
		printf("Best Chromosome so far is %f\n",m_BestFitness);
	}

#ifdef STAT
	CStatistics::Get()->PrintStats();
#endif
}