#ifndef __CCONATINER_H__
#define __CCONATINER_H__

#include "CChromosome.h"
#include "Geom.h"
#include "CPolyDisplay.h"

class CContainer;

class CContainerCallback{
public:
	virtual bool OnGenerationDone( int nGeneration, const CContainer *pContainer ) = 0;	
};

enum{
	ID_GENERATION_DONE = 200,
};

class CContainer {
private:

	CPolyDisplay		*m_pDisplay;
	wxFrame				*m_pCallback;
	bool				 m_bStop;

	int RandomEventCore(float p[],int n);
	int RandomEvent(float p[],int n,int ex);
	
public:
	float m_Width;
	float m_Height;
	float m_WorstHeight;

	float m_AvgFitness;

	std::vector< CPolygon >		m_PolyList;
	std::vector< CChromosome >	m_ChromosomePool;

	CChromosome					m_BestChromosome;
	float						m_BestFitness;
	float						m_WorstFitness;

	float						m_CullThreshold;

	CContainer( float Width, float Height )
	{
		m_Width  = Width;
		m_Height = Height;
		m_WorstHeight = 0;

		m_CullThreshold = 0.7f;

		m_pDisplay = NULL;
		m_pCallback = NULL;
		
		m_bStop = false;
	}

	~CContainer();

	void SetDisplay( CPolyDisplay *pDisplay )
	{
		m_pDisplay = pDisplay;
	}

	void SetCallback( wxFrame *pCallback )
	{
		m_pCallback = pCallback;
	}

	
	void  GetCollidablePolygonList( const CPolygon &Poly, const std::vector< CPolygon > &PolyPool, std::vector< CPolygon > &Output );

	void  ValidateChromosome( CChromosome &c );
	bool  AddPolygon( const char *FileName );
	
	void  Clear()
	{
		m_PolyList.clear();
		m_ChromosomePool.clear();
	}

	void  PrepareEverything( unsigned int nFirstPopulationSize );
	void  RandomlyGenerateFirstGeneration();
	float EvaluateFitnessValue( const CChromosome &c, std::vector<CPolygon> &DumpedPolyList, bool bDrawDebugPoints = false);
	void  RandomlyGenerateChromosome( CChromosome &c );

	void RunGA( int nFirstPopulationSize, int nGenerations );

	float GetAvgFitness()
	{
		return m_AvgFitness;
	}

	float GetBestFitness()
	{
		return m_BestFitness;
	}

	size_t GetTotalPopulation()
	{
		return m_ChromosomePool.size();
	}

	void Stop()
	{
		m_bStop = true;
	}

	void OnGenerationDone( int i )
	{
		if( !m_pCallback ) return;
		wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED,ID_GENERATION_DONE);
		e.SetInt(i);
		wxPostEvent(m_pCallback,e);
	}
};

#endif