#ifndef __CPOLYDISPLAY_H__
#define __CPOLYDISPLAY_H__

#include <wx/wx.h>
#include <wx/glcanvas.h>
#include <vector>
#include <wx/xrc/xmlres.h>

#include <FTGL.h>
#include <FTGLPixmapFont.h>

#include "Geom.h"

extern int g_OpenGLAttribList[];

struct SPolyOptions{
	bool		m_bDrawAnchorPoint;
	bool		m_bDrawBoundingBox;
	bool		m_bDrawFrontLines;
	bool		m_bDrawBackLines;
	bool		m_bDrawFrontPoints;
	bool		m_bDrawBackPoints;
	bool		m_bDrawSeq;
};

struct SPolyItem{
	CPolygon	m_Polygon;
	int			m_Seq;


	SPolyItem()
	{
		/*
		m_bDrawAnchorPoint = false;
		m_bDrawBoundingBox = false;
		m_bDrawFrontLines  = false;
		m_bDrawBackLines   = false;
		m_bDrawFrontPoints = false;
		m_bDrawBackPoints  = false;
		m_bDrawSeq		   = false;
		*/
	}
};

#define LEFT_MARGIN 40
#define BOTTOM_MARGIN 20

extern SPolyOptions g_PolyOptions;

class CPolyDisplay : public wxGLCanvas {
private:
	std::vector< SPolyItem > m_PolyList;
	std::vector< mgVector3 > m_DebugPointList;

	float					 m_LogicalWidth;
	float					 m_LogicalHeight;
	float					 m_Width;
	float					 m_Height;
	float					 m_MaxY;

	FTGLPixmapFont			 m_Font;

	wxMenu					 *m_pRenderMenu;

	void DrawPolygon( SPolyItem& PolyItem );
	
public:
	CPolyDisplay( wxWindow *pParent ) 
		:wxGLCanvas( pParent, wxID_ANY, wxPoint(0,0), wxSize(400,600),0,_T("GLCanvas"),
		g_OpenGLAttribList) , m_Font("arial.ttf")

	{
		m_pRenderMenu = NULL;
	}

	void SetupView()
	{
		int w,h;
		SetCurrent();

		GetClientSize(&w,&h);
		glViewport(0,0,w,h);
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		m_LogicalHeight = ((float)h/(float)w) * m_LogicalWidth;
		glOrtho(0,m_LogicalWidth,0,m_LogicalHeight,-1,1);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}

	void Initialize( float Width, float Height )
	{
		Show(true);

		SetCurrent();

		glDisable(GL_DEPTH_TEST);
		glShadeModel(GL_FLAT);
				

		glClearColor(1,1,1,0);

		m_Width = Width;
		m_Height = Height;

		m_LogicalWidth  = Width+LEFT_MARGIN*2;
		m_LogicalHeight = Height+BOTTOM_MARGIN;

		SetupView();

		glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);		
		glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);

		glEnable(GL_POINT_SMOOTH);

		glClear(GL_COLOR_BUFFER_BIT);

		m_Font.FaceSize(12);

		wxMenuBar *pMenuBar = wxXmlResource::Get()->LoadMenuBar(_T("RenderMenu"));
		
		if( pMenuBar ){
			int i = pMenuBar->FindMenu(_T("Options"));
			if( i != wxNOT_FOUND ){
				m_pRenderMenu = pMenuBar->GetMenu(i);
			}
		}

		if( m_pRenderMenu ){
			
			wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_BOUNDING_BOX"));
			if( pItem ){
				pItem->Check( false );
			}

			pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_FRONT_POINTS"));
			if( pItem ){
				pItem->Check( false );
			}

			pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_BACK_POINTS"));
			if( pItem ){
				pItem->Check( false );
			}

			pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_REFERENCE"));
			if( pItem ){
				pItem->Check( false );
			}

			pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_SEQ_NUMBER"));
			if( pItem ){
				pItem->Check( false );
			}			
		}
	}
	
	void OnPaint( wxPaintEvent& e );
	void Clear()
	{
		m_PolyList.clear();
		m_MaxY = 0;
	}
	void AddPolygon( CPolygon &Poly );
	
	void ClearDebugPointList()
	{
		m_DebugPointList.clear();
	}
	void AddDebugPoint( const mgVector3 &v )
	{
		m_DebugPointList.push_back( v );
	}

	void AddDebugPoint( float x, float y, float z )
	{
		AddDebugPoint( mgVector3(x,y,z) );
	}

	void DrawAxes();

	void OnEraseBackground( wxEraseEvent &e )
	{
	}

	void OnRightDown( wxMouseEvent &e );
	
	void OnMenuDrawBoundingBox( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawBoundingBox = e.IsChecked();
		
		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_BOUNDING_BOX"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}
/*
	EVT_MENU(XRCID("ID_RENDER_DRAW_BOUNDING_BOX"),CPolyDisplay::OnMenuDrawBoundingBox)
	EVT_MENU(XRCID("ID_RENDER_DRAW_FRONT_POINTS"),CPolyDisplay::OnMenuDrawFrontPoints)
	EVT_MENU(XRCID("ID_RENDER_DRAW_BACK_POINTS"),CPolyDisplay::OnMenuDrawBackPoints)	
	EVT_MENU(XRCID("ID_RENDER_DRAW_REFERENCE"),CPolyDisplay::OnMenuDrawReferencePoints)		
*/
	void OnMenuDrawFrontPoints( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawFrontPoints = e.IsChecked();
		
		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_FRONT_POINTS"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}

	void OnMenuDrawBackPoints( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawBackPoints = e.IsChecked();
		
		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_BACK_POINTS"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}
	
	void OnMenuDrawReferencePoints( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawAnchorPoint = e.IsChecked();
		
		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_REFERENCE"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}
	void OnMenuDrawSeqNumber( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawSeq = e.IsChecked();

		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_SEQ_NUMBER"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}

	void OnMenuDrawFrontLines( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawFrontLines = e.IsChecked();

		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_FRONT_LINES"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}

	void OnMenuDrawBackLines( wxCommandEvent &e )
	{
		g_PolyOptions.m_bDrawBackLines = e.IsChecked();

		wxMenuItem *pItem = m_pRenderMenu->FindItem(XRCID("ID_RENDER_DRAW_BACK_LINES"),NULL);
		if( pItem ) pItem->Check( e.IsChecked() );
		
		Refresh();
	}
	DECLARE_EVENT_TABLE();
};

class CPlotter: public wxGLCanvas{
private:
	std::vector< mgVector3 > m_PointList;
	float					 m_MaxY;
	float					 m_MinY;
	float					 m_TopY;

	float					 m_Steps;
	float					 m_Width;
	float					 m_Height;

	float					 m_XStep;
	float					 m_YStep;

	static FTGLPixmapFont	 m_Font;
	static bool				 m_bFirst;
	static float			 m_LeftMargin;
	static float			 m_RightMargin;
	static float			 m_BottomMargin;
	static float			 m_TopMargin;

	//float					 m_LogicalWidth;
	//float					 m_LogicalHeight;

public:
	CPlotter(wxWindow *pParent);

	void AddPoint( float x,float y)
	{
		
		if( !m_PointList.size() ){
			m_MaxY = y;
			m_MinY = y;
			Resize( m_Steps );
		}

		if( m_MaxY < y ) m_MaxY = y;		
		

		if( (m_MaxY >= m_TopY) ||( m_MinY > y) ){
			
			if(m_MinY > y) 
				m_MinY = y;

			if( (m_MaxY >= m_TopY) ) 
				m_TopY = (m_MaxY-m_MinY) * 1.2f + m_MinY;
						
			Resize( m_Steps );
		}

		m_PointList.push_back( mgVector3(x,y,0) );
	}

	void Resize(float Steps)
	{
		int w,h;
		GetClientSize( &w,&h );

		SetCurrent();

		glViewport(0,0,w,h);
		
		m_Steps = Steps;
		m_Width = w;
		m_Height = h;

		//m_LogicalWidth  = m_Width + m_LeftMargin + m_RightMargin;
		//m_LogicalHeight = m_TopY + m_BottomMargin + m_TopMargin;

		m_XStep = (w - m_LeftMargin - m_RightMargin) / m_Steps;
		m_YStep = (h - m_BottomMargin - m_TopMargin) / (m_TopY - m_MinY);

		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		glOrtho(0,w,0,h,-1,1);

		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
	}

	void Initialize( float Width  )
	{
		Show(true);

		SetCurrent();

		glDisable(GL_DEPTH_TEST);
		glShadeModel(GL_FLAT);
					
		glClearColor(1,1,1,0);

		m_Width = Width;		

		Resize( Width );

		glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);
		glHint(GL_LINE_SMOOTH_HINT,GL_NICEST);
		
		glClear(GL_COLOR_BUFFER_BIT);

		m_Font.FaceSize(8);
	}

	void Clear()
	{
		m_PointList.clear();
		m_TopY = 50;
		m_MaxY = 0;	
		m_MinY = 0;
	}

	void OnPaint( wxPaintEvent &e );
	
	void OnEraseBackground( wxEraseEvent &e )
	{
	}

	DECLARE_EVENT_TABLE();
};

#endif
