#include "Geom.h"
#include "CStat.h"

const mgVector3 MG_VECTOR3_X = mgVector3(1,0,0);
const mgVector3 MG_VECTOR3_Y = mgVector3(0,1,0);
const mgVector3 MG_VECTOR3_Z = mgVector3(0,0,1);

bool DoesPointHitUpward( const mgVector3 &Point, const mgVector3 &LineStart, const mgVector3 &LineEnd, const mgVector3 &LineParallel, float *yResult = NULL)
{

#ifdef STAT
	CStatistics::Get()->AddCollision( 1 );
#endif

	if( ((Point.m_v[X] >= LineStart.m_v[X]) && (Point.m_v[X] <= LineEnd.m_v[X]) ) ||
		((Point.m_v[X] >= LineEnd.m_v[X]) && (Point.m_v[X] <= LineStart.m_v[X]) ) )
	{
		if( LineParallel.m_v[X] != 0.0 ){
			
			float Grad = LineParallel.m_v[Y] / LineParallel.m_v[X];		
			
			float yIntercept = Grad * ( Point.m_v[X] - LineStart.m_v[X]) + LineStart.m_v[Y];
			
			if( yIntercept >= Point.m_v[Y] ){ 
				if( yResult ) *yResult = yIntercept;
				return true;
			}
			else 
				return false;

		}
		else{
			float MaxY = LineStart.m_v[Y] > LineEnd.m_v[Y] ? LineStart.m_v[Y] : LineEnd.m_v[Y];

			if( Point.m_v[Y] <= MaxY ){
				if( yResult ) {
					
					float MinY = LineStart.m_v[Y] < LineEnd.m_v[Y] ? LineStart.m_v[Y] : LineEnd.m_v[Y];
					
					if( Point.m_v[Y] > MinY ) *yResult = MinY;
					else *yResult = Point.m_v[Y];
				}
				return true;
			}
			else
				return false;			
		}
	}
	else 
		return false;
}

bool DoesPointHitDownward( const mgVector3 &Point, const mgVector3 &LineStart, const mgVector3 &LineEnd, const mgVector3 &LineParallel, float *yResult = NULL)
{

#ifdef STAT
	CStatistics::Get()->AddCollision( 1 );
#endif

	if( ((Point.m_v[X] >= LineStart.m_v[X]) && (Point.m_v[X] <= LineEnd.m_v[X]) ) ||
		((Point.m_v[X] >= LineEnd.m_v[X]) && (Point.m_v[X] <= LineStart.m_v[X]) ) )
	{
		if( LineParallel.m_v[X] != 0.0 ){
			
			float Grad = LineParallel.m_v[Y] / LineParallel.m_v[X];		
			
			float yIntercept = Grad * ( Point.m_v[X] - LineStart.m_v[X]) + LineStart.m_v[Y];
			
			if( yIntercept <= Point.m_v[Y] ){
				if( yResult ) *yResult = yIntercept;
				return true;
			}
			else 
				return false;

		}
		else{
			float MinY = LineStart.m_v[Y] < LineEnd.m_v[Y] ? LineStart.m_v[Y] : LineEnd.m_v[Y];

			if( Point.m_v[Y] >= MinY ){
				if( yResult ) {
					float MaxY = LineStart.m_v[Y] > LineEnd.m_v[Y] ? LineStart.m_v[Y] : LineEnd.m_v[Y];
					
					if( Point.m_v[Y] >= MaxY ) 
						*yResult = MaxY;
					else 
						*yResult = Point.m_v[Y];					
				}
				return true;
			}
			else
				return false;			
		}
	}
	else 
		return false;
}


bool CPolygon::DoesPointHitAnyLineUpward(int PointIdx)
{
	for(size_t l=0;l<m_LineList.size();++l){
			
		if( (m_PointList[PointIdx].m_InLineIdx == l) ||
			(m_PointList[PointIdx].m_OutLineIdx == l) )
		{
			continue;
		}

		if( DoesPointHitUpward( m_PointList[ PointIdx ].m_Point,
			m_PointList[ m_LineList[l].m_FromPointIdx ].m_Point,
			m_PointList[ m_LineList[l].m_ToPointIdx ].m_Point,
			m_LineList[l].m_Line) )
		{
			return true;			
		}
	}

	return false;
}

bool CPolygon::DoesPointHitAnyLineDownward(int PointIdx)
{
	for(size_t l=0;l<m_LineList.size();++l){
			
		if( (m_PointList[PointIdx].m_InLineIdx == l) ||
			(m_PointList[PointIdx].m_OutLineIdx == l) )
		{
			continue;
		}

		if( DoesPointHitDownward( m_PointList[ PointIdx ].m_Point,
			m_PointList[ m_LineList[l].m_FromPointIdx ].m_Point,
			m_PointList[ m_LineList[l].m_ToPointIdx ].m_Point,
			m_LineList[l].m_Line) )
		{
			return true;			
		}
	}

	return false;
}

void CPolygon::FilterFrontAndBackLines()
{
	std::vector< CLine > m_TmpFrontLines;
	std::vector< CLine > m_TmpBackLines;

	for(size_t i=0;i<m_FrontFacingLines.size();++i){

		if( !DoesPointHitAnyLineDownward( m_FrontFacingLines[i].m_FromPointIdx ) ||
			!DoesPointHitAnyLineDownward( m_FrontFacingLines[i].m_ToPointIdx ) )
		{
			m_TmpFrontLines.push_back( m_FrontFacingLines[i] );
		}
	}

	for(size_t i=0;i<m_BackFacingLines.size();++i){

		if( !DoesPointHitAnyLineUpward( m_BackFacingLines[i].m_FromPointIdx ) ||
			!DoesPointHitAnyLineUpward( m_BackFacingLines[i].m_ToPointIdx ) )
		{
			m_TmpBackLines.push_back( m_BackFacingLines[i] );
		}
	}

	m_FrontFacingLines = m_TmpFrontLines;
	m_BackFacingLines  = m_TmpBackLines;
}

void CPolygon::FilterFrontAndBackPoints()
{
	bool bPointPassed;
	
	std::vector< CPoint > m_TmpFrontPointList;
	std::vector< CPoint > m_TmpBackPointList;

	for(size_t i=0;i<m_FrontFacingCollidablePoints.size();++i){

		bPointPassed = true;

		for(size_t l=0;l<m_LineList.size();++l){
			
			if( (m_FrontFacingCollidablePoints[i].m_InLineIdx == l) ||
				(m_FrontFacingCollidablePoints[i].m_OutLineIdx == l) )
			{
				continue;
			}

			if( DoesPointHitDownward( m_FrontFacingCollidablePoints[i].m_Point,
				m_PointList[ m_LineList[l].m_FromPointIdx ].m_Point,
				m_PointList[ m_LineList[l].m_ToPointIdx ].m_Point,
				m_LineList[l].m_Line) )
			{
				bPointPassed = false;
				break;
			}
		}

		if( bPointPassed )
			m_TmpFrontPointList.push_back( m_FrontFacingCollidablePoints[i]);
	}

	for(size_t i=0;i<m_BackFacingCollidablePoints.size();++i){

		bPointPassed = true;


		for(size_t l=0;l<m_LineList.size();++l){
			
			if( (m_BackFacingCollidablePoints[i].m_InLineIdx == l) ||
				(m_BackFacingCollidablePoints[i].m_OutLineIdx == l) )
			{
				continue;
			}

			if( DoesPointHitUpward( m_BackFacingCollidablePoints[i].m_Point,
				m_PointList[ m_LineList[l].m_FromPointIdx ].m_Point,
				m_PointList[ m_LineList[l].m_ToPointIdx ].m_Point,
				m_LineList[l].m_Line) )
			{
				bPointPassed = false;
				break;
			}
		}

		if( bPointPassed )
			m_TmpBackPointList.push_back( m_BackFacingCollidablePoints[i]);
	}

	m_FrontFacingCollidablePoints = m_TmpFrontPointList;
	m_BackFacingCollidablePoints = m_TmpBackPointList;
}

void CPolygon::ProcessFrontAndBackLines()
{
	size_t nLines = m_LineList.size();
	
	for( size_t i = 0; i< nLines; ++i ){
		mgVector3 vResult;
		
		m_LineList[i].m_Line.CrossProduct( (mgVector3)MG_VECTOR3_Y, vResult );
		
		if( vResult.m_v[Z] >= 0 ) {
			/* Front Facing */
			m_LineList[i].m_FrontFacing = true;
			m_FrontFacingLines.push_back( m_LineList[i] ); 
		}
		else{
			/* Back Facing */
			m_LineList[i].m_FrontFacing = false;
			m_BackFacingLines.push_back( m_LineList[i] );
		}
	}
}


void CPolygon::ProcessFrontAndBackPoints()
{
	size_t nPoints = m_PointList.size();
	
	for( size_t i=0;i<nPoints;++i){

		if (  (m_LineList[ m_PointList[ i ].m_InLineIdx ].m_FrontFacing && 
			  !m_LineList[ m_PointList[ i ].m_OutLineIdx].m_FrontFacing) 
			  ||
			  (!m_LineList[ m_PointList[ i ].m_InLineIdx ].m_FrontFacing && 
			  m_LineList[ m_PointList[ i ].m_OutLineIdx].m_FrontFacing) 
			  )
		{
			// If the point is in the junction of back-face lines and 
			// front-face lines then simply add it 
			
			m_PointList[i].m_BackFacingCollidable = true;
			m_PointList[i].m_FrontFacingCollidable = true;

			m_FrontFacingCollidablePoints.push_back( m_PointList[i] );
			m_BackFacingCollidablePoints.push_back( m_PointList[i] );
		}
		else if(m_LineList[ m_PointList[ i ].m_InLineIdx ].m_FrontFacing && 
			    m_LineList[ m_PointList[ i ].m_OutLineIdx].m_FrontFacing)
		{
			//if both in-line and out-line are front facing
			
			if( (m_LineList[ m_PointList[i].m_InLineIdx ].m_Line.DotProduct( (mgVector3)MG_VECTOR3_Y )) <= 
				(m_LineList[ m_PointList[i].m_OutLineIdx ].m_Line.DotProduct( (mgVector3)MG_VECTOR3_Y )) )
			{
				m_PointList[i].m_FrontFacingCollidable = true;
				m_PointList[i].m_BackFacingCollidable = false;
				m_FrontFacingCollidablePoints.push_back( m_PointList[i] );
			}
		}
		else {

			// if both in-line and out-line are back facing/
			if( (m_LineList[ m_PointList[i].m_InLineIdx ].m_Line.DotProduct( (mgVector3)MG_VECTOR3_Y )) >= 
				(m_LineList[ m_PointList[i].m_OutLineIdx ].m_Line.DotProduct( (mgVector3)MG_VECTOR3_Y )) )
			{
				m_PointList[i].m_FrontFacingCollidable = false;
				m_PointList[i].m_BackFacingCollidable = true;

				m_BackFacingCollidablePoints.push_back( m_PointList[i] );
			}
		}

	}
}



void CPolygon::BuildConnectivity()
{
	size_t nPoints = m_PointList.size();

	for(size_t i=0;i<nPoints;++i){

		m_LineList.push_back( CLine() );

		m_LineList[i].m_FromPointIdx = (int)i;
		m_LineList[i].m_ToPointIdx = (int)((i + 1) % nPoints);
		
		m_LineList[i].m_Line = m_PointList[m_LineList[i].m_ToPointIdx].m_Point - m_PointList[ m_LineList[i].m_FromPointIdx ].m_Point;

		m_PointList[ m_LineList[i].m_FromPointIdx ].m_OutLineIdx = (int)i;
		m_PointList[ m_LineList[i].m_ToPointIdx ].m_InLineIdx = (int)i;
	}
}

void CPolygon::Process()
{
	BuildConnectivity();
	ProcessFrontAndBackLines();
	ProcessFrontAndBackPoints();

	FilterFrontAndBackPoints();
	FilterFrontAndBackLines();
}

bool CPolygon::ReadPolygon( const char *pFileName )
{
	FILE *fp = fopen( pFileName, "r");
	
	if( !fp ){
		printf("Couldn't read polyon file %s\n", pFileName);
		return false;
	}

	int nPoints;
	fscanf(fp,"%d",&nPoints);
	
	for(int i=0;i<nPoints;++i){
		
		float x,y;
		
		fscanf( fp, "%f,%f",&x,&y);
		
		m_PointList.push_back( CPoint(x,y) );
	}
	
	float maxx,maxy, minx,miny;

	fscanf(fp,"\n;%f,%f\n%f,%f",&maxx,&maxy,&minx,&miny);

	m_BoundingBox.SetBoundingBox( maxx,maxy,minx,miny );

	fclose( fp );

	Process();

	return true;
}

bool CPolygon::GetApproachDistanceFromTop( CPolygon &Poly, float *pDistance, float *pXPoint, float *pYPoint )
{
	float yDistance;
	bool  bFirstCalc = true;
	float SmallestLength = 0;
	bool  bCollided = false;

	for(size_t i=0;i<m_FrontFacingCollidablePoints.size();++i){
		
		for(size_t n=0;n<Poly.m_BackFacingLines.size();++n){
			
			mgVector3 Point = m_FrontFacingCollidablePoints[i].m_Point + m_Pos;
			mgVector3 StartPoint = Poly.m_PointList[Poly.m_BackFacingLines[n].m_FromPointIdx].m_Point + Poly.m_Pos;
			mgVector3 ToPoint = Poly.m_PointList[Poly.m_BackFacingLines[n].m_ToPointIdx].m_Point + Poly.m_Pos;

			if( DoesPointHitDownward(Point,
				StartPoint,
				ToPoint,
				Poly.m_BackFacingLines[n].m_Line,
				&yDistance ))
			{
				bCollided = true;

				if(bFirstCalc ){
					bFirstCalc = false;					
					SmallestLength = m_Pos.m_v[Y] + m_FrontFacingCollidablePoints[i].m_Point[Y] - yDistance;
					if( pYPoint ) *pYPoint = yDistance;
					if( pXPoint ) *pXPoint = Point.m_v[X];
				}
				else{
					float l = m_Pos.m_v[Y] + m_FrontFacingCollidablePoints[i].m_Point[Y] - yDistance;
					
					//SmallestLength = l < SmallestLength ? l : SmallestLength;
					
					if( SmallestLength > l ){
						SmallestLength = l;
						if( pYPoint ) *pYPoint = yDistance;
						if( pXPoint ) *pXPoint = Point.m_v[X];
					}
				}
			}
		}
	}


	for(size_t i=0;i<Poly.m_BackFacingCollidablePoints.size();++i){
		
		for(size_t n=0;n<m_FrontFacingLines.size();++n){
			
			mgVector3 Point = Poly.m_BackFacingCollidablePoints[i].m_Point + Poly.m_Pos;
			mgVector3 StartPoint = m_PointList[m_FrontFacingLines[n].m_FromPointIdx].m_Point + m_Pos;
			mgVector3 ToPoint = m_PointList[m_FrontFacingLines[n].m_ToPointIdx].m_Point + m_Pos;

			if( DoesPointHitUpward(Point,
				StartPoint,
				ToPoint,
				m_FrontFacingLines[n].m_Line,
				&yDistance ))
			{
				bCollided = true;

				if(bFirstCalc ){
					bFirstCalc = false;					
					SmallestLength = yDistance - Poly.m_BackFacingCollidablePoints[i].m_Point[Y] - Poly.m_Pos.m_v[Y];
					if( pYPoint ) *pYPoint = Point.m_v[Y];//yDistance;
					if( pXPoint ) *pXPoint = Point.m_v[X];
				}
				else{
					float l = yDistance - Poly.m_BackFacingCollidablePoints[i].m_Point[Y] - Poly.m_Pos.m_v[Y];
					//SmallestLength = l < SmallestLength ? l : SmallestLength;
					
					if( SmallestLength > l ){
						SmallestLength = l;
						if( pYPoint ) *pYPoint = Point.m_v[Y];//yDistance;
						if( pXPoint ) *pXPoint = Point.m_v[X];
					}
				}
			}
		}
	}

	if( pDistance ) *pDistance = SmallestLength;

	return bCollided;
}

