#ifndef __GEOM_H__
#define __GEOM_H__

#include <vector>

#include "mgVector.h"

class CPoint {
public:
	int			m_InLineIdx;
	int			m_OutLineIdx;
	
	mgVector3	m_Point;
	
	bool		m_FrontFacingCollidable;
	bool		m_BackFacingCollidable;

	CPoint()
	{
		m_Point.SetVector( 0, 0, 0 );
		
		m_FrontFacingCollidable = false;
		m_BackFacingCollidable = false;

		m_InLineIdx  = 0;
		m_OutLineIdx = 0;
	}

	CPoint( float x, float y )
	{
		m_Point.SetVector( x, y, 0 );
		
		m_FrontFacingCollidable = false;
		m_BackFacingCollidable = false;

		m_InLineIdx  = 0;
		m_OutLineIdx = 0;
	}

	void SetPoint( float x, float y )
	{
		m_Point.SetVector( x, y, 0 );
	}
};


class CLine {
public:
	int			m_FromPointIdx;
	int			m_ToPointIdx;
	
	bool		m_FrontFacing;

	mgVector3	m_Line;

	CLine()
	{
		m_FromPointIdx = 0;
		m_ToPointIdx = 0;
		m_Line = mgVector3(0,0,0);
		m_FrontFacing = false;
	}
};

class CBoundingBox {
public:
	mgVector3	m_MinPoint;
	mgVector3	m_MaxPoint;
	
	CBoundingBox()
	{
		m_MinPoint = mgVector3(0,0,0);
		m_MaxPoint = mgVector3(0,0,0);
	}

	bool IsCollide( CBoundingBox &Other )
	{
		return !(
		(m_MaxPoint.m_v[X] < Other.m_MinPoint.m_v[X]) ||
		(m_MinPoint.m_v[X] > Other.m_MaxPoint.m_v[X]) ||
		
		(m_MaxPoint.m_v[Y] < Other.m_MinPoint.m_v[Y]) ||
		(m_MinPoint.m_v[Y] > Other.m_MaxPoint.m_v[Y]) 
		);
	}

	void SetBoundingBox( float MaxX, float MaxY, float MinX,float MinY )
	{
		m_MinPoint.SetVector( MinX, MinY, 0 );
		m_MaxPoint.SetVector( MaxX, MaxY, 0 );
	}

	float GetHeight()
	{
		return m_MaxPoint.m_v[Y] - m_MinPoint.m_v[Y];
	}
};

class CPolygon {
private:
	
	bool DoesPointHitAnyLineDownward(int PointIdx);
	bool DoesPointHitAnyLineUpward(int PointIdx);

public:
	std::vector< CPoint >	m_PointList;
	std::vector< CLine  >	m_LineList;
	CBoundingBox			m_BoundingBox;

	std::vector< CLine  >	m_FrontFacingLines;
	std::vector< CLine	>   m_BackFacingLines;
	
	std::vector< CPoint >   m_FrontFacingCollidablePoints;
	std::vector< CPoint >	m_BackFacingCollidablePoints;

	mgVector3				m_Pos;

	CPolygon()
	{
		m_Pos = mgVector3(0,0,0);
	}

	void	FilterFrontAndBackPoints();
	void	FilterFrontAndBackLines();
	void	BuildConnectivity();
	void	ProcessFrontAndBackLines();
	void	ProcessFrontAndBackPoints();

	void	Process();
	bool	ReadPolygon( const char *pFileName );
	
	bool	GetApproachDistanceFromTop( CPolygon &Poly , float *pDistance = NULL, float *pXPoint = NULL,float *pYPoint = NULL);
};

#endif

