#include "PolyFitMain.h"
#include "CPolyDisplay.h"
#include "CContainer.h"

BEGIN_EVENT_TABLE(CPolyFitFrame,wxFrame)
	EVT_MENU(XRCID("ID_MNU_COMMANDS_NEXT"),CPolyFitFrame::OnMenuNext)
	EVT_BUTTON(XRCID("ID_BTN_ADD"),CPolyFitFrame::OnAdd)
	EVT_BUTTON(XRCID("ID_BTN_CLEAR"),CPolyFitFrame::OnClear)
	EVT_BUTTON(XRCID("ID_BTN_REMOVE"),CPolyFitFrame::OnRemove)
	EVT_BUTTON(XRCID("ID_BTN_RUN_GA"),CPolyFitFrame::OnRunGA)
	EVT_BUTTON(XRCID("ID_BTN_STOP"),CPolyFitFrame::OnStop)
	EVT_BUTTON(XRCID("ID_BTN_SHOW_RESULT"),CPolyFitFrame::OnShowResult)
	EVT_MENU(ID_GENERATION_DONE,CPolyFitFrame::OnGenerationDone)
	EVT_COMMAND_SCROLL(XRCID("ID_SLIDER_MUTATION"),CPolyFitFrame::OnMutationChange) 	
	EVT_COMMAND_SCROLL(XRCID("ID_SLIDER_CULL_THRESHOLD"),CPolyFitFrame::OnCullThresholdChange)			
END_EVENT_TABLE()

BEGIN_EVENT_TABLE( CPolyFitScrolledWindow,wxScrolledWindow)	
END_EVENT_TABLE()

CPolyFitFrame::CPolyFitFrame( const wxString &strTitle ) : 
	wxFrame( (wxFrame*) NULL, wxID_ANY, strTitle,wxPoint(0,0) ), m_Container(400,600)
{
	m_pGauge = NULL;

	wxXmlResource::Get()->InitAllHandlers();

	if( !wxXmlResource::Get()->Load( _T("PolyFit_wdr.xrc") ) ){
		wxMessageBox( _T("Couldn't load the XML GUI resource"), _T("Error"), wxOK, this );
	}

	//SetMenuBar( wxXmlResource::Get()->LoadMenuBar(_T("PolyFitMainMenu")) );

	wxBoxSizer *pMainSizer   =  new wxBoxSizer( wxVERTICAL );
	wxBoxSizer *pScrollSizer =  new wxBoxSizer( wxVERTICAL );

	/**** Loading the main GUI ****/
	wxScrolledWindow *pScrolledWindow = new CPolyFitScrolledWindow(this, this);	
	wxPanel *pMainPanel = wxXmlResource::Get()->LoadPanel( pScrolledWindow,_T("PolyFitMainDlg") );		

	pScrollSizer->Add( pMainPanel,1,wxEXPAND | wxALL,1 );	
	pScrolledWindow->SetSizer( pScrollSizer );
	pScrollSizer->SetSizeHints( pScrolledWindow );
	pScrolledWindow->SetScrollbars(10,10,100,100);
	pScrolledWindow->EnableScrolling(true,true);

	
	pMainSizer->Add( pScrolledWindow, 1, wxEXPAND | wxALL, 1 );
	
	SetSizer( pMainSizer );
	pMainSizer->SetSizeHints( this );

	m_pScrolledWindow = pScrolledWindow;

	m_pPolyThread = NULL;

	Maximize(true);
}

void CPolyFitFrame::OnGenerationDone( wxCommandEvent &e )
{
	
	int nGeneration = e.GetInt();
	
	m_pGauge->SetValue( (nGeneration * m_pGauge->GetRange() )/ m_nGenerations );
	
	m_pAvgFitness->SetValue( wxString::Format( _T("%f"), m_Container.GetAvgFitness() ) );
	m_pBestFitness->SetValue( wxString::Format(_T("%f"),  m_Container.GetBestFitness() ) );
	m_pTotalPop->SetValue( wxString::Format( _T("%d"), m_Container.GetTotalPopulation() ) );

	m_pAvgPlotter->AddPoint( nGeneration, m_Container.GetAvgFitness() );
	m_pBestPlotter->AddPoint( nGeneration, m_Container.GetBestFitness() );
	m_pTotalPopPlotter->AddPoint( nGeneration, m_Container.GetTotalPopulation() );

	m_pAvgPlotter->Refresh();
	m_pBestPlotter->Refresh();
	m_pTotalPopPlotter->Refresh();

	//wxGetApp().Yield();	

	//printf("OnGeneration %d\n",e.GetInt());	
}


void CPolyFitFrame::OnScrolled()
{
	//if(m_pDisplay) m_pDisplay->Refresh();
	printf("Scrolled\n");
}


void CPolyFitFrame::OnCullThresholdChange( wxScrollEvent &e )
{
	wxSlider   *pSliderCull = (wxSlider*)FindWindow(XRCID("ID_SLIDER_CULL_THRESHOLD") );
	wxTextCtrl *pTextCtrl       = (wxTextCtrl*)FindWindow(XRCID("ID_TXT_THRESHOLD") );

	float Value = (float)(pSliderCull->GetValue() - pSliderCull->GetMin()) / (float)(pSliderCull->GetMax() - pSliderCull->GetMin());
	pTextCtrl->SetValue( wxString::Format(_T("%1.2f"),Value) );
}

void CPolyFitFrame::OnMutationChange( wxScrollEvent &e )
{
	wxSlider   *pSliderMutation = (wxSlider*)FindWindow(XRCID("ID_SLIDER_MUTATION") );
	wxTextCtrl *pTextCtrl       = (wxTextCtrl*)FindWindow(XRCID("ID_TXT_MUTATION_PROBABILITY") );

	float Value = (float)(pSliderMutation->GetValue() - pSliderMutation->GetMin()) / (float)(pSliderMutation->GetMax() - pSliderMutation->GetMin());
	pTextCtrl->SetValue( wxString::Format(_T("%1.2f"),Value) );

	g_MutationProbability = Value; // Dyanamically change the mutation probability.
}

void CPolyFitFrame::OnStop( wxCommandEvent &e )
{
	m_Container.Stop();
	if( m_pPolyThread ){
		m_pPolyThread->Delete();
		delete m_pPolyThread;
		m_pPolyThread = NULL;
	}
}

void CPolyFitFrame::OnShowResult( wxCommandEvent &e )
{
	std::vector< CPolygon > DumpedPolyList;
	m_Container.EvaluateFitnessValue(m_Container.m_BestChromosome,DumpedPolyList);

	m_pDisplay->Clear();
	for(size_t i=0;i<DumpedPolyList.size();++i){
		m_pDisplay->AddPolygon( DumpedPolyList[i] );
	}

	printf("Best Chromosome so far is %f\n",m_Container.m_BestChromosome.m_FitnessValue);

	m_pDisplay->Refresh();	
}

void CPolyFitFrame::OnRemove( wxCommandEvent &e )
{	
	wxListBox *pList = (wxListBox*)FindWindow(XRCID("ID_LST_POLYGON") );
	int Sel = pList->GetSelection(  );
	
	if( Sel != wxNOT_FOUND )
		pList->Delete( Sel );
}

void CPolyFitFrame::OnClear( wxCommandEvent &e )
{
	wxListBox *pList = (wxListBox*)FindWindow(XRCID("ID_LST_POLYGON") );
	pList->Clear();
}

void CPolyFitFrame::OnAdd( wxCommandEvent &e )
{
	wxFileDialog dlg(this, _T("Choose a polygon file"), _T("./"), _T(""),_T("Polygon File (*.pol)|*.pol"),wxOPEN | wxMULTIPLE);
	if( dlg.ShowModal() == wxID_OK ){
		wxListBox *pList = (wxListBox*)FindWindow(XRCID("ID_LST_POLYGON") );
		
		wxArrayString FileNameList;
		dlg.GetPaths( FileNameList );
		
		pList->InsertItems( FileNameList, pList->GetCount() );
	}
}

void CPolyFitFrame::OnRunGA( wxCommandEvent &e )
{

	if( m_pPolyThread && m_pPolyThread->IsRunning() ){
		printf("Algorithm thread is already running\n");
		return;
	}

	wxSpinCtrl *pSpinInitialPop       = (wxSpinCtrl*)FindWindow(XRCID("ID_SPIN_INITIAL_POPULATION") );
	wxSpinCtrl *pSpinGenerations      = (wxSpinCtrl*)FindWindow(XRCID("ID_SPIN_GENERATIONS") );
	wxSlider   *pSliderMutation       = (wxSlider*)FindWindow(XRCID("ID_SLIDER_MUTATION") );
	//wxSlider   *pSliderCullThreshold  = (wxSlider*)FindWindow(XRCID("ID_SLIDER_CULL_THRESHOLD") );

	wxListBox  *pList = (wxListBox*)FindWindow(XRCID("ID_LST_POLYGON") );
	
	//m_Container.m_CullThreshold = (float)(pSliderCullThreshold->GetValue() - pSliderCullThreshold->GetMin()) / (float)(pSliderCullThreshold->GetMax() - pSliderCullThreshold->GetMin());
	g_MutationProbability = (float)(pSliderMutation->GetValue() - pSliderMutation->GetMin()) / (float)(pSliderMutation->GetMax() - pSliderMutation->GetMin());

	m_Container.Clear();
	for(int i=0;i<pList->GetCount();++i){
		wxString Str = pList->GetString( i );

#ifndef wxUSE_UNICODE		
			if(!m_Container.AddPolygon( (char*) Str.c_str() ))
				printf("Couldn't Load Polygon %s\n",Str.c_str() );
#else
			if(!m_Container.AddPolygon(  Str.ToAscii() ))
				printf("Couldn't Load Polygon %s\n",Str.ToAscii());		
#endif

	}
	
	m_nGenerations = pSpinGenerations->GetValue();

	m_pAvgPlotter->Clear();
	m_pBestPlotter->Clear();
	m_pTotalPopPlotter->Clear();

	m_pAvgPlotter->Resize( m_nGenerations );
	m_pBestPlotter->Resize( m_nGenerations );
	m_pTotalPopPlotter->Resize( m_nGenerations );

	if( pList->GetCount() ){
		m_Container.SetDisplay( m_pDisplay );
		m_Container.SetCallback( this  );
		//m_Container.RunGA(pSpinInitialPop->GetValue(), pSpinGenerations->GetValue());
		
		if(m_pPolyThread) delete m_pPolyThread;

		m_pPolyThread = new CPolyThread();
		m_pPolyThread->SetMainFrame(this);
		m_pPolyThread->SetPopulatonSize( pSpinInitialPop->GetValue() );
		m_pPolyThread->SetGenerations( pSpinGenerations->GetValue() );
		m_pPolyThread->Create();
		
		wxThreadError e =  m_pPolyThread->Run();
		printf("Thread Run returned: %d\n",e);
	}
	
}

void CPolyFitFrame::Initialize()
{		
	srand( time(NULL ) );
	
	/**** Attaching the OpenGL display ****/	
	
	m_pDisplay = new CPolyDisplay( m_pScrolledWindow );
	wxXmlResource::Get()->AttachUnknownControl(_T("ID_DISPLAY"), m_pDisplay , m_pScrolledWindow);

	m_pDisplay->Initialize(400,600);	
	m_pDisplay->Refresh();

	m_pAvgPlotter = new CPlotter( m_pScrolledWindow );
	wxXmlResource::Get()->AttachUnknownControl(_T("ID_AVG_PLOTTER"), m_pAvgPlotter , m_pScrolledWindow);

	m_pAvgPlotter->Initialize(150);	
	m_pAvgPlotter->Refresh();

    m_pBestPlotter = new CPlotter( m_pScrolledWindow );
	wxXmlResource::Get()->AttachUnknownControl(_T("ID_BEST_PLOTTER"), m_pBestPlotter , m_pScrolledWindow);

	m_pBestPlotter->Initialize(150);	
	m_pBestPlotter->Refresh();

	m_pTotalPopPlotter = new CPlotter( m_pScrolledWindow );
	wxXmlResource::Get()->AttachUnknownControl(_T("ID_TOTAL_POP_PLOTTER"), m_pTotalPopPlotter , m_pScrolledWindow);

	m_pTotalPopPlotter->Initialize(150);	
	m_pTotalPopPlotter->Refresh();
	

	m_pGauge = (wxGauge*)FindWindow(XRCID("ID_GAUGE") );
	
	m_pAvgFitness	= (wxTextCtrl*) FindWindow( XRCID("ID_TXT_AVG_FITNESS")) ;
	m_pBestFitness	= (wxTextCtrl*) FindWindow( XRCID("ID_TXT_BEST_FITNESS")) ;
	m_pTotalPop		= (wxTextCtrl*) FindWindow( XRCID("ID_TXT_TOTAL_POP")) ;
	
}

void CPolyFitFrame::OnMenuNext( wxCommandEvent &e )
{
	static int i = 0;
	/*
	CChromosome &c = m_Container.m_ChromosomePool[i++];
	i = i % m_Container.m_ChromosomePool.size();

	m_Container.ValidateChromosome( c );

	std::vector< CPolygon > DumpedPolyList;
	float f = m_Container.EvaluateFitnessValue( c, DumpedPolyList );

	printf("%f\n",f);

	*/

}

bool CPolyFitApp::OnInit()
{
	CPolyFitFrame *pFrame = new CPolyFitFrame( _T("Polygon Fit using GA") );
	
	pFrame->Show(true);
	pFrame->Initialize();

	return true;
}


wxThread::ExitCode CPolyThread::Entry()
{
	printf("Inside the entry code\n");
	m_pMainFrame->m_Container.RunGA(m_nPopulation, m_nGenerations);
	printf("Thread Exited\n");	

	wxMessageBox(_T("Genetic Algorithm Thread has ended"),_T("PolyFit"),wxOK|wxICON_INFORMATION);

	return 0;
}
