#include <wx/wx.h>
#include <wx/spinctrl.h>
#include <wx/image.h>
#include <wx/xrc/xmlres.h>

#include "CPolyDisplay.h"
#include "CContainer.h"

#define ID_SCROLLEDWINDOW 100

class CPolyFitFrame;

class CPolyThread : public wxThread {
public:
	CPolyFitFrame *m_pMainFrame;
	int			   m_nPopulation;
	int			   m_nGenerations;

	CPolyThread(  ): wxThread(wxTHREAD_JOINABLE)
	{
		m_pMainFrame = NULL;
		m_nPopulation = 0;
		m_nGenerations = 0;
	}

	void SetMainFrame( CPolyFitFrame *pFrame )
	{
		m_pMainFrame = pFrame;
	}

	void SetPopulatonSize( int p )
	{
		if( !IsRunning() ){
			m_nPopulation = p;
		}else{
			printf("Thread is already running\n");
		}
	}

	void SetGenerations( int p )
	{
		if( !IsRunning() ){
			m_nGenerations = p;
		}else{
			printf("Thread is already running\n");
		}
	}
	
	virtual ExitCode Entry();
};

class CScrollCallback {
public:
	virtual void OnScrolled() = 0;
};

class CPolyFitFrame: public wxFrame, CScrollCallback{
protected:
	CPolyDisplay	 *m_pDisplay;
	
	CPlotter		 *m_pAvgPlotter;
	CPlotter		 *m_pBestPlotter;
	CPlotter		 *m_pTotalPopPlotter;

	wxScrolledWindow *m_pScrolledWindow;
	CContainer		 m_Container;
	int				 m_nGenerations;
	wxGauge			*m_pGauge;
	wxTextCtrl		*m_pAvgFitness;
	wxTextCtrl		*m_pBestFitness;
	wxTextCtrl		*m_pTotalPop;

	CPolyThread		*m_pPolyThread;

public:
	
	CPolyFitFrame( const wxString& strTitle );
	
	~CPolyFitFrame()
	{
		if(m_pPolyThread) delete m_pPolyThread;
	}

	void Initialize();

	void OnMenuNext( wxCommandEvent& e );
	void OnAdd( wxCommandEvent& e );
	void OnClear( wxCommandEvent &e );
	void OnRemove( wxCommandEvent &e );
	void OnStop( wxCommandEvent &e );
	void OnRunGA( wxCommandEvent &e );
	void OnShowResult( wxCommandEvent &e );
	void OnMutationChange( wxScrollEvent &e );
	void OnCullThresholdChange( wxScrollEvent &e );
	void OnScrolled();
	void OnGenerationDone( wxCommandEvent &e );

	friend class CPolyThread;

	DECLARE_EVENT_TABLE()
};

class CPolyFitScrolledWindow : public wxScrolledWindow{
private:
	CScrollCallback *m_pCallback;
public:
	CPolyFitScrolledWindow( wxWindow *pParent, CScrollCallback *pCallback = 0 ) :
	  wxScrolledWindow( pParent )
	{
		m_pCallback = pCallback;
	}	

	DECLARE_EVENT_TABLE();
};

class CPolyFitApp : public wxApp{
public:
	virtual bool OnInit();
};

IMPLEMENT_APP(CPolyFitApp)
DECLARE_APP(CPolyFitApp)


