/************************************************************
	
	mgVector3.h
	-------------
	Author: Shams
	Revised and Corrected by: Zahid	

	This is a sole property of Mirage project 

*************************************************************/

#ifndef MG_VECTOR_H
#define MG_VECTOR_H

// Disabling double to float conversion warning
// for VC++
#if defined(WIN32) || defined(_WIN32)
#pragma warning(disable:4305)
#endif

#include <math.h>

#include "mgDef.h"
#include <memory.h>

enum eVectorOrd{
	X,Y,Z,W
};

enum ePlaneOrg{
	A,B,C,D
};

class MG_DLLAPI mgVector3
{
public :
	
	mgReal_t m_v[ 3 ] ;

	mgVector3()
	{
		m_v[ 0 ] = m_v[ 1 ] = m_v[ 2 ] = 0 ;
	} ;

	// Added by Zahid
	mgVector3(mgReal_t i,mgReal_t j,mgReal_t k)
	{
		m_v[ 0 ] = i;  
		m_v[ 1 ] = j;
		m_v[ 2 ] = k;
	}

	void SetVector( mgReal_t i,mgReal_t j,mgReal_t k)
	{
		m_v[ 0 ] = i;  
		m_v[ 1 ] = j;
		m_v[ 2 ] = k;
	}

	mgVector3( mgReal_t data[ 3 ] )
	{
		m_v[ 0 ] = data[ 0 ] ;  
		m_v[ 1 ] = data[ 1 ] ;
		m_v[ 2 ] = data[ 2 ] ; 
	} ;

	void Add( mgVector3 &operand, mgVector3 &result ) 
	{
		result.m_v[ 0 ] = m_v[ 0 ] + operand.m_v[ 0 ] ; 
		result.m_v[ 1 ] = m_v[ 1 ] + operand.m_v[ 1 ] ; 
		result.m_v[ 2 ] = m_v[ 2 ] + operand.m_v[ 2 ] ; 
	} ;

	void Subtract( mgVector3 &operand, mgVector3 &result ) 
	{
		result.m_v[ 0 ] = m_v[ 0 ] - operand.m_v[ 0 ] ; 
		result.m_v[ 1 ] = m_v[ 1 ] - operand.m_v[ 1 ] ; 
		result.m_v[ 2 ] = m_v[ 2 ] - operand.m_v[ 2 ] ; 
	} ;
	
	mgReal_t DotProduct( const mgVector3 &operand ) 
	{
		return ( m_v[ 0 ] * operand.m_v[ 0 ] ) +
			   ( m_v[ 1 ] * operand.m_v[ 1 ] ) +
			   ( m_v[ 2 ] * operand.m_v[ 2 ] ) ;
	} ;

	void CrossProduct( const mgVector3 &operand, mgVector3 &result )
	{
		result.m_v[0] = m_v[1]*operand.m_v[2] - m_v[2]*operand.m_v[1];
		result.m_v[1] = m_v[2]*operand.m_v[0] - m_v[0]*operand.m_v[2];
		result.m_v[2] = m_v[0]*operand.m_v[1] - m_v[1]*operand.m_v[0];
	} ;

	mgReal_t Projection( mgVector3 &operand ) 
	{
		mgReal_t d = operand.Length();
		if(!d) return 0;

		return DotProduct( operand ) / d ;
	} ;

	mgReal_t LengthSquare()  
	{
		return ( m_v[ 0 ] * m_v[ 0 ] ) 
			 + ( m_v[ 1 ] * m_v[ 1 ] )
			 + ( m_v[ 2 ] * m_v[ 2 ] ) ;
	} ;

	void Normalize()
	{
		mgReal_t d = Length();
		if(!d) return;

		m_v[0] /= d;
		m_v[1] /= d;
		m_v[2] /= d;
	}

	mgReal_t Length() 
	{
		return (mgReal_t) sqrt( LengthSquare() ) ;
	} ;	

	// Added by Zahid
	mgVector3 operator + (const mgVector3& other) const 
	{
		return mgVector3(m_v[0] + other.m_v[0],
			             m_v[1] + other.m_v[1],
						 m_v[2] + other.m_v[2] );
	}

	mgVector3 operator - ( const mgVector3& other) const
	{
		return mgVector3(m_v[0] - other.m_v[0],
			             m_v[1] - other.m_v[1],
						 m_v[2] - other.m_v[2] );
	}

	mgVector3 operator * (mgReal_t k) 
	{
		return mgVector3(m_v[0] * k,
			             m_v[1] * k,
						 m_v[2] * k);
	}

	mgVector3 operator / (mgReal_t k)  
	{
		return mgVector3(m_v[0] / k,
			             m_v[1] / k,
						 m_v[2] / k);
	}

	mgReal_t& operator [](int i)
	{
		return m_v[i];
	}

	void CalculateRightUpVector(mgVector3 &r,mgVector3 &u)
	{
		mgReal_t sx = SIGN(m_v[X]);
		mgReal_t sy = SIGN(m_v[X]);
		mgReal_t sz = SIGN(m_v[X]);

		mgReal_t x = ABS(m_v[X]);
		mgReal_t y = ABS(m_v[Y]);
		mgReal_t z = ABS(m_v[Z]);

		if((x > EPSILON) && (y<EPSILON) && (z < EPSILON )){
			r.m_v[X] = 0;
			r.m_v[Y] = 0;
			r.m_v[Z] = 1*sx;

			u.m_v[X] = 0;
			u.m_v[Y] = 1;
			u.m_v[Z] = 0;
			return;
		}

		if((y > EPSILON) && (x<EPSILON) && (z<EPSILON)){
			r.m_v[X] = 0;
			r.m_v[Y] = 0;
			r.m_v[Z] = -1 * sy;

			u.m_v[X] = 1;
			u.m_v[Y] = 0;
			u.m_v[Z] = 0;
			return;
		}

		if((z > EPSILON) && (x<EPSILON) && (y < EPSILON)){
			r.m_v[X] = -1*sz;
			r.m_v[Y] = 0;
			r.m_v[Z] = 0;

			u.m_v[X] = 0;
			u.m_v[Y] = 1;
			u.m_v[Z] = 0;
			return;
		}

		mgVector3 majorAxis;
	
		if( (x >= y) && (x >= z)) {
			majorAxis.m_v[X] = 1;
			majorAxis.m_v[Y] = 0;
			majorAxis.m_v[Z] = 0;		
		}
		else if((y >= x) && (y >= z)){
			majorAxis.m_v[X] = 0;
			majorAxis.m_v[Y] = 1;
			majorAxis.m_v[Z] = 0;
		}
		else{
			majorAxis.m_v[X] = 0;
			majorAxis.m_v[Y] = 0;
			majorAxis.m_v[Z] = 1;
		}
	
		majorAxis.CrossProduct((*this),r);
		CrossProduct(r,u);

		r.Normalize();
		u.Normalize();
	}
};

// Added by Zahid
class MG_DLLAPI mgVector4 : public mgVector3{
public:
	mgReal_t m_w;

	mgVector4(){
		m_w = 0;
	}
	mgVector4(mgReal_t i,mgReal_t j,mgReal_t k,mgReal_t l) :
	          mgVector3(i,j,k)
	{
		m_w = l;
	}

	mgVector4(mgReal_t data[4]) : 
	          mgVector3(data)
	{
		m_w = data[3];
	}

	void Add(mgVector4 &operand,mgVector4 &result)
	{
		mgVector3::Add(operand,result);
		result.m_w = m_w + operand.m_w;
	}
	void Subtract(mgVector4 &op,mgVector4 &res)
	{
		mgVector3::Subtract(op,res);
		res.m_w = m_w - op.m_w;
	}
	
	mgVector4 operator + (mgVector4 &op)
	{
		return mgVector4( m_v[0] + op.m_v[0],
			              m_v[1] + op.m_v[1],
						  m_v[2] + op.m_v[2],
						  m_w    + op.m_w
						);
	}

	mgVector4 operator - (mgVector4 &op)
	{
		return mgVector4( m_v[0] - op.m_v[0],
			              m_v[1] - op.m_v[1],
						  m_v[2] - op.m_v[2],
						  m_w    - op.m_w
						);
	}

	mgReal_t& operator [](int i){
		if(i<3) 
			return mgVector3::operator[](i);
		else 
			return m_w;
	}
};

class mgPlane{
public:
	mgReal_t m_Plane[4];

	mgPlane(){
		memset(m_Plane,0,sizeof(m_Plane));
	}	


	void SetPlane(mgVector3 &normal,mgReal_t distance)
	{
		m_Plane[0] = normal[0];
		m_Plane[1] = normal[1];
		m_Plane[2] = normal[2];
		m_Plane[3] = -distance;
	}

	void CalculatePlane(mgVector3 vList[3])
	{
		mgVector3 v0,v1,n;

		vList[1].Subtract(vList[0],v0);
		vList[2].Subtract(vList[1],v1);		

		v0.CrossProduct(v1,n);

		n.Normalize();
		mgReal_t d = n.DotProduct(vList[0]);

		SetPlane(n,d);
	}

	void GetNormal(mgVector3& norm)
	{
		norm[0] = m_Plane[0];
		norm[1] = m_Plane[1];
		norm[2] = m_Plane[2];
	}

	void Normalize()
	{
		mgReal_t d = (mgReal_t)sqrt(m_Plane[0] * m_Plane[0] +
			                        m_Plane[1] * m_Plane[1] +
					                m_Plane[2] * m_Plane[2]);
		m_Plane[0] /= d;
		m_Plane[1] /= d;
		m_Plane[2] /= d;
		m_Plane[3] /= d;
	}
	
	mgReal_t Evaluate(mgVector3 &v)
	{
		return m_Plane[0] * v[0] +
			   m_Plane[1] * v[1] +
			   m_Plane[2] * v[2] +
			   m_Plane[3];
	}
};

extern const mgVector3 MG_VECTOR3_X;
extern const mgVector3 MG_VECTOR3_Y;
extern const mgVector3 MG_VECTOR3_Z;

#endif 

